<?php

namespace Drupal\virtual_base\Form;

use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\virtual_base\VirtualBaseManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Virtual base form settings.
 *
 * @SuppressWarnings(PHPMD.CamelCaseParameterName)
 * @SuppressWarnings(PHPMD.CamelCaseVariableName)
 * @SuppressWarnings(PHPMD.LongClassName)
 * @SuppressWarnings(PHPMD.LongVariable)
 */
class VirtualBaseSettingsForm extends ConfigFormBase {
  /**
   * Drupal\virtual_base\VirtualBaseManagerInterface definition.
   *
   * @var \Drupal\virtual_base\VirtualBaseManagerInterface
   */
  protected $helper;

  /**
   * Module Handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Container.
   *
   * @var \Symfony\Component\DependencyInjection\ContainerInterface
   */
  protected $container;

  /**
   * Constructs a SecKitSettingsForm object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\virtual_base\VirtualBaseManagerInterface $helper
   *   Helper.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   Module handler.
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   Container.
   */
  public function __construct(ConfigFactoryInterface $config_factory, VirtualBaseManagerInterface $helper, ModuleHandlerInterface $module_handler, ContainerInterface $container) {
    parent::__construct($config_factory);
    $this->helper = $helper;
    $this->moduleHandler = $module_handler;
    $this->container = $container;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('virtual_base.manager'),
      $container->get('module_handler'),
      $container
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'virtual_base_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['virtual_base.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('virtual_base.settings');

    $form['enabled'] = [
      '#type' => 'checkbox',
      '#default_value' => $config->get('enabled'),
      '#title' => $this->t('Enabled'),
      '#return_value' => TRUE,
      '#description' => $this->t('Enable a virtual base.'),
    ];

    $form['path_prefix'] = [
      '#type' => 'textfield',
      '#maxlength' => 254,
      '#default_value' => $config->get('path_prefix'),
      '#title' => $this->t('Path Prefix'),
      '#description' => $this->t("Specify the path prefix to be the virtual base. it should starts with a /"),
      '#states' => [
        'required' => [
          'input[name="enabled"]' => ['checked' => TRUE],
        ],
        'visible' => [
          'input[name="enabled"]' => ['checked' => TRUE],
        ],
      ],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // If From-Origin is enabled, it should be explicitly set.
    $enabled = $form_state->getValue("enabled");
    $path_prefix = $form_state->getValue("path_prefix");
    if ($enabled) {
      if (!str_starts_with($path_prefix, "/")) {
        $form_state->setErrorByName("path_prefix", $this->t("Should start with a /"));
      }
      if (str_ends_with($path_prefix, "/")) {
        $form_state->setErrorByName("path_prefix", $this->t("Should not end with a /"));
      }
      if (UrlHelper::encodePath($path_prefix) !== $path_prefix) {
        $form_state->setErrorByName("path_prefix", $this->t("Should be properly encoded"));
      }
      // Avoid conflict with path alias if already set.
      // When the request hit this path we cannot know
      // if it's the home with path prefix or the path alias requested.
      if ($this->moduleHandler->moduleExists('path_alias')) {
        $url = Url::fromUri('internal:' . $path_prefix);
        if ($url->isRouted()) {
          $params = $url->getRouteParameters();
          $entity_type = key($params);
          if (!empty($params)) {
            $form_state->setErrorByName("path_prefix", $this->t("The path prefix is already set as path alias of %entity_type %id.", [
              '%entity_type' => $entity_type,
              '%id' => $params[$entity_type],
            ]));
          }
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('virtual_base.settings');
    $config->set("enabled", $form_state->getValue("enabled") ? TRUE : FALSE);
    $config->set("path_prefix", $form_state->getValue("path_prefix") ?? "");
    $config->save();
    /*
    $this->container->getDefinition('virtual_base.path_processor')
    ->clearTag('path_processor_inbound')
    ->clearTag('path_processor_outbound');*/
    parent::submitForm($form, $form_state);
  }

}
