<?php

namespace Drupal\virtual_base;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Config\ConfigFactoryInterface;

/**
 * Helper service for the Virtual Base Module.
 *
 *  @SuppressWarnings(PHPMD.CamelCaseParameterName)
 *  @SuppressWarnings(PHPMD.CamelCaseVariableName)
 *  @SuppressWarnings(PHPMD.LongClassName)
 *  @SuppressWarnings(PHPMD.LongVariable)
 */
class VirtualBaseManager implements VirtualBaseManagerInterface {

  /**
   * Drupal\Core\Config\ConfigFactoryInterface definition.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Cache backend.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cacheBackend;

  public function __construct(
    ConfigFactoryInterface $config_factory,
    CacheBackendInterface $cacheBackend,
  ) {
    $this->configFactory = $config_factory;
    $this->cacheBackend = $cacheBackend;
  }

  /**
   * Get the config (no caching).
   *
   * @return \Drupal\Core\Config\ImmutableConfig
   *   Config.
   */
  protected function getConfigImpl() {
    return $this->configFactory->get('virtual_base.settings');
  }

  /**
   * Get the config (with caching).
   */
  public function getConfig() {
    $cid = 'virtual_base:settings';
    $staticCache = &drupal_static(__FUNCTION__ . $cid, NULL);
    if ($staticCache) {
      // If the static cache exists, then return it.
      return $staticCache;
    }
    // Get the cache out of the database and return the data component.
    $result = $this->cacheBackend->get($cid);
    if ($result && $result->data !== NULL) {
      return $result->data;
    }

    $data = $this->getConfigImpl();

    // Store cache.
    $tags = [
      'config:virtual_base.settings',
    ];
    $this->cacheBackend->set($cid, $data, CacheBackendInterface::CACHE_PERMANENT, $tags);
    return $data;
  }

  /**
   * {@inheritDoc}
   */
  public function resolveInboundPath($path, $request = NULL): string {
    $config = $this->getConfig();
    $prefix = $config->get("path_prefix");
    // Virtual base not enabled.
    // @todo DEAL WITH PATH ALIAS before ?!
    if (!$config->get("enabled") || strlen($prefix) < 1 || !str_starts_with($path, $prefix)) {
      return $path;
    }
    // @todo what happen if no request, need to create it ?
    if ($request) {
      $request->query->set("_virtual_base", $prefix);
    }
    return substr($path, strlen($prefix));
  }

  /**
   * {@inheritDoc}
   */
  public function resolveOutboundPath($path, $request = NULL): string {

    $config = $this->getConfig();
    if (!$request || !$config->get("enabled")) {
      return $path;
    }
    $prefix = $config->get("path_prefix");
    if (strlen($prefix) > 1 && $request->query->get("_virtual_base") === $prefix) {
      return $prefix . $path;
    }
    return $path;
  }

}
